%% Solution for Exercise 3: Newton’s method for optimization
%    TEMPO Summer School on Numerical Optimal Control and Embedded Optimization 
%    University of Freiburg, July 27 - August 7, 2015 
%    Rien Quirynen, Dimitris Kouzoupis and Moritz Diehl 

%% Part 0: Plotting Rosenbrock’s function

clc;
clear all;
close all;

% Create figure
figure1 = figure('Position',[1 400 1200 600]);
colormap('gray');
axis square;
R=0:.002:sqrt(2);
TH=2*pi*(0:.002:1); 
X=R'*cos(TH); 
Y=R'*sin(TH); 
Z=log(1+vrosenbrock(X,Y));

% Create subplot
subplot1 = subplot(1,2,1,'Parent',figure1);
view([124 34]);
grid('on');
hold('all');

% Create surface
surf(X,Y,Z,'Parent',subplot1,'LineStyle','none');

% Create contour
contour(X,Y,Z,'Parent',subplot1);

% Create subplot
subplot2 = subplot(1,2,2,'Parent',figure1);
view([234 34]);
grid('on');
hold('all');

% Create surface
surf(X,Y,Z,'Parent',subplot2,'LineStyle','none');

% Create contour
contour(X,Y,Z,'Parent',subplot2);


%% Part 1: Implementing Newton's method
z = [0; 0];
res = 1; iter = 0;
while norm(res) > 1e-5
    [~,res,H] = D2f_eval(z);
    dz = -H\res
    z = z + dz;
    iter = iter+1;
end
disp(['solution found in ' num2str(iter) ' iterations: x = ' num2str(z(1)) ', y = ' num2str(z(2))])


%% Part 2: Solving the problem using fminunc
% fminunc's solution:
options = optimoptions(@fminunc,'Display','iter','Algorithm','quasi-newton');
x_sol = fminunc(@f_eval,[0 0],options)

% fminunc's solution with gradient and hessian:
options = optimoptions(@fminunc,'Display','iter','Algorithm','trust-region','GradObj','on','Hessian','on');
x_sol1 = fminunc(@D2f_eval,[0 0],options)


%% Part 2: Solving the constrained problem using fmincon
options = optimoptions(@fmincon,'Display','iter', ...
    'Algorithm','interior-point','GradObj','on','GradConstr','on','Hessian','bfgs');
x_sol2 = fmincon(@Df_eval,[0 0],[],[],[],[],[],[],@c_eval,options)

options = optimoptions(@fmincon,'Display','iter', ...
    'Algorithm','interior-point','GradObj','on','GradConstr','on','Hessian','user-supplied','HessFcn',@hessian_fun);
x_sol3 = fmincon(@Df_eval,[0 0],[],[],[],[],[],[],@c_eval,options)

